# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli.jc_api_v1.py
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

This is a utility library for the jumpcloud version 1 api

.. note::

    To learn more about the jumpcloud api 1
    `project website <https://github.com/TheJumpCloud/jcapi-python/tree/master/jcapiv1>`_.
"""
from distutils.util import strtobool

import jcapiv1
from jcapiv1.rest import ApiException

from jccli.helpers import class_to_dict

# pylint: disable=too-many-arguments
class JumpcloudApiV1:
    """
        Wrapper for Jumpcloud API v1
    """
    def __init__(self, api_key):
        configuration = jcapiv1.Configuration()
        configuration.api_key['x-api-key'] = api_key
        self.system_users_api = jcapiv1.SystemusersApi(jcapiv1.ApiClient(configuration))

    def get_users(self, limit=10, search='', filter='', sort='', fields=''):
        """
        Get users from jumpcloud
        :param limit:
        :param skip:
        :param search:
        :param filter:
        :param sort:
        :param fields:
        :return: a list of users with dict of settings
        """
        content_type = 'application/json'
        accept = 'application/json'
        skip = 0
        x_org_id = ''

        try:
            api_response = self.system_users_api.systemusers_list(content_type,
                                                                  accept,
                                                                  limit=limit,
                                                                  skip=skip,
                                                                  sort=sort,
                                                                  fields=fields,
                                                                  x_org_id=x_org_id,
                                                                  search=search,
                                                                  filter=filter)
            users = class_to_dict(api_response.results)
            return users
        except ApiException as error:
            raise "Exception when calling SystemusersApi->systemusers_list: %s\n" % error

    def create_user(self, systemuser):
        """
        Create a new user in jumpcloud
        :param systemuser: a dictoionary of Systemuser properties
               https://github.com/TheJumpCloud/jcapi-java/blob/master/jcapiv1/docs/Systemuser.md
        :return: The api response
        """
        content_type = 'application/json'
        accept = 'application/json'
        x_org_id = ''
        body = jcapiv1.Systemuserputpost(username=systemuser['username'],
                                         email=systemuser['email'],
                                         firstname=systemuser.get('firstname', ''),
                                         lastname=systemuser.get('lastname', ''),
                                         account_locked=strtobool(
                                             systemuser.get('account_locked', 'False')),
                                         activated=strtobool(
                                             systemuser.get('activated', 'False')),
                                         allow_public_key=strtobool(
                                             systemuser.get('allow_public_key', 'True')),
                                         ldap_binding_user=strtobool(
                                             systemuser.get('ldap_binding_user', 'False')),
                                         passwordless_sudo=strtobool(
                                             systemuser.get('passwordless_sudo', 'False')),
                                         sudo=strtobool(systemuser.get('sudo', 'False')))
        try:
            api_response = self.system_users_api.systemusers_post(content_type,
                                                                  accept,
                                                                  body=body,
                                                                  x_org_id=x_org_id)
            return api_response
        except ApiException as error:
            raise "Exception when calling SystemusersApi->systemusers_post: %s\n" % error


    def delete_user(self, user_id):
        """
        Delete a user from jumpcloud
        :param id: The jumpcloud id of the user
        :return:
        """
        content_type = 'application/json'
        accept = 'application/json'
        x_org_id = ''
        try:
            api_response = self.system_users_api.systemusers_delete(user_id,
                                                                    content_type,
                                                                    accept,
                                                                    x_org_id=x_org_id)
            return api_response
        except ApiException as error:
            raise "Exception when calling SystemusersApi->systemusers_post: %s\n" % error

    def get_user_id(self, username):
        """
        Get the jumpcloud user id from the user name
        :param username
        :return:  the user id
        """
        users = self.get_users(limit='', fields="username")

        user_id = None
        for user in users:
            if user['_username'] == username:
                user_id = user['_id']

        return user_id



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli.cli
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

This is the entry point for the command-line interface (CLI) application.  It
can be used as a handy facility for running the task from a command line.

.. note::

    To learn more about Click visit the
    `project website <http://click.pocoo.org/5/>`_.  There is also a very
    helpful `tutorial video <https://www.youtube.com/watch?v=kNke39OZ2k0>`_.

    To learn more about running Luigi, visit the Luigi project's
    `Read-The-Docs <http://luigi.readthedocs.io/en/stable/>`_ page.
"""
import logging
import click
from jccli.errors import SystemUserNotFoundError, MissingRequiredArgumentError

from jccli.helpers import get_users_from_file, get_user_from_file, get_user_from_term
from jccli.helpers import get_groups_from_file
from jccli.jc_api_v1 import JumpcloudApiV1
from jccli.jc_api_v2 import JumpcloudApiV2
from .__init__ import __version__

LOGGING_LEVELS = {
    0: logging.NOTSET,
    1: logging.ERROR,
    2: logging.WARN,
    3: logging.INFO,
    4: logging.DEBUG,
}  #: a mapping of `verbose` option counts to logging levels


class Info():
    """
    An information object to pass data between CLI functions.
    """

    def __init__(self):  # Note: This object must have an empty constructor.
        self.verbose: int = 0


# pass_info is a decorator for functions that pass 'info' objects.
#: pylint: disable=invalid-name
pass_info = click.make_pass_decorator(Info, ensure=True)


@click.group()
@click.option('--key', "-k", required=False, type=str, help='Jumpcloud API key')
@click.option("--verbose", "-v", count=True, help="Enable verbose output.")
@pass_info
def cli(info, key, verbose: int):
    """
    Run jccli.
    """
    # Use the verbosity count to determine the logging level...
    if verbose > 0:
        logging.basicConfig(
            level=LOGGING_LEVELS[verbose]
            if verbose in LOGGING_LEVELS
            else logging.DEBUG
        )
        click.echo(
            click.style(
                f"Verbose logging is enabled. "
                f"(LEVEL={logging.getLogger().getEffectiveLevel()})",
                fg="yellow",
            )
        )
    info.verbose = verbose
    info.key = key

@cli.command()
def version():
    """
    Get the version.
    """
    click.echo(click.style(f"{__version__}", bold=True))


@cli.command()
@click.option('--json', "-j", required=False, type=str,
              help='SystemUser properties json')
@click.option('--file', "-f", required=False, type=click.Path(exists=True),
              help='SystemUser properties file')
@pass_info
def create_user(info, json, file):
    """
    Create a new Jumpcloud user
    """
    api1 = JumpcloudApiV1(info.key)
    user = {}
    if json is not None:
        user = get_user_from_term(json)
    elif file is not None:
        user = get_user_from_file(file)
    else:
        raise MissingRequiredArgumentError("SystemUser properties not provided")

    click.echo("Create jumpcloud user " + user['username'])
    response = api1.create_user(user)
    click.echo(response)

@cli.command()
@click.option('--name', "-n", required=True, type=str, help='Name of the group')
@click.option('--type', "-t", required=True, type=click.Choice(['user', 'system']),
              help='The type of group')
@pass_info
def create_group(info, name, type):
    """
    Create a Jumpcloud group
    """
    api2 = JumpcloudApiV2(info.key)
    # click.echo("Create jumpcloud {} group {}".format(type, name))
    response = api2.create_group(name, type)
    click.echo(
        click.style(
            f"{response}",
            fg="green",
        )
    )

@cli.command()
@click.option('--name', "-n", required=True, type=str, help='Name of the group')
@pass_info
def delete_group(info, name):
    """
    Delete a Jumpcloud group
    """
    api2 = JumpcloudApiV2(info.key)
    response = api2.delete_group(name)
    if response is None:
        click.echo("Jumpcloud group deleted")

@cli.command()
@click.option('--username', "-u", required=False, type=str, help='The user name')
@pass_info
def delete_user(info, username):
    """
    Delete a jumpcloud user
    """
    if username is None:
        raise MissingRequiredArgumentError("Deleting a user requires a username")

    api1 = JumpcloudApiV1(info.key)
    click.echo("Delete jumpcloud user " + username)
    id = api1.get_user_id(username)
    if id is None:
        raise SystemUserNotFoundError("System user {} not found".format(username))

    response = api1.delete_user(id)
    click.echo(response)


@cli.command()
@click.option('--data', "-d", required=True, type=click.Path(exists=True),
              help='The JC data file')
@pass_info
def sync(info, data):
    """
    Sync Jumpcloud with a data file
    """
    click.echo("Sync data on jumpcloud with data in " + data)
    users = get_users_from_file(data)
    groups = get_groups_from_file(data)
    if users is None:
        raise MissingRequiredArgumentError("A sync requires a config file")

    sync_users(info.key, users)


def sync_users(key, users):
# pylint: disable-msg=too-many-locals
# pylint: disable-msg=too-many-branches
    """
    Sync data file with jumpcloud
    """
    api1 = JumpcloudApiV1(key)
    api2 = JumpcloudApiV2(key)

    jc_usernames = []
    jc_emails = []
    jc_users = []
    jc_users_request = api1.get_users(limit='')
    if jc_users_request:
        for jc_user in jc_users_request:
            jc_usernames.append(jc_user['_username'])
            jc_emails.append(jc_user['_email'])
            jc_users.append({'username':jc_user['_username'], 'email':jc_user['_email']})

    click.echo("jumpcloud users: " + ','.join(jc_usernames))

    # create new users
    added_users = []
    for user in users:
        do_create_user = False
        try:
            user_name = user['username']
            user_email = user['email']
            if (user_name not in jc_usernames) and (user_email not in jc_emails):
                do_create_user = True
            else:
                click.echo(user_name + " user already exists")
        except KeyError as e:
            raise e

        if do_create_user:
            click.echo("creating user: " + user_name)
            # response = api1.create_user(user_name,
            #                             user_email,
            #                             user['firstname'],
            #                             user['lastname'])
            # if response is not None:
            #     added_users.append({'username':user_name, 'email':user_email})
            added_users.append({'username':user_name, 'email':user_email})
            group_id = api2.get_group("staff")
            if group_id:
                user_id = api1.get_user_id(user_name)
                click.echo("binding " + user_id + " to group: " + group_id)
                # api2.bind_user_to_group(user_id, group_id)

    # remove users that do not exist in the users file
    local_usernames = []
    local_emails = []
    for user in users:
        local_usernames.append(user['username'])
        local_emails.append(user['email'])

    removed_users = []
    for jc_user in jc_users:
        do_remove_user = False
        user_name = jc_user['username']
        user_email = jc_user['email']
        if (user_name not in local_usernames) and (user_email not in local_emails):
            do_remove_user = True

        if do_remove_user:
            click.echo("removing user: " + user_name)
            removed_users.append(jc_user)



# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli.errors.py
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

Exceptions

"""

class SystemUserNotFoundError(Exception):
    """
    Jumpcloud system user is not found
    """

class UerGroupNotFoundError(Exception):
    """
    Jumpcloud user group is not found
    """

class MissingRequiredArgumentError(Exception):
    """
    Required arguments are missing
    """



# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli.jc_api_v2.py
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

This is a utility library for the jumpcloud version 2 api

.. note::

    To learn more about the jumpcloud api 2
    `project website <https://github.com/TheJumpCloud/jcapi-python/tree/master/jcapiv2>`_.
"""
import jcapiv2
from jcapiv2.rest import ApiException

from jccli.helpers import class_to_dict
from jccli.errors import UerGroupNotFoundError

class JumpcloudApiV2:
    """
        Wrapper for Jumpcloud API v2
    """
    def __init__(self, api_key):
        configuration = jcapiv2.Configuration()
        configuration.api_key['x-api-key'] = api_key
        self.graph_api = jcapiv2.GraphApi(jcapiv2.ApiClient(configuration))
        self.groups_api = jcapiv2.GroupsApi(jcapiv2.ApiClient(configuration))
        self.user_groups_api = jcapiv2.UserGroupsApi(jcapiv2.ApiClient(configuration))
        self.system_groups_api = jcapiv2.SystemGroupsApi(jcapiv2.ApiClient(configuration))
        self.bulk_job_requests_api = jcapiv2.BulkJobRequestsApi(jcapiv2.ApiClient(configuration))

    def create_group(self, name, type):
        """
        Create a Jumpcloud group
        :param name: The group name
        :param type: The group type
        :return: API response
        """
        group_name = name
        group_type = type
        content_type = 'application/json'
        accept = 'application/json'
        x_org_id = ''

        if group_type == "system":
            try:
                body = jcapiv2.SystemGroupData(name=group_name)
                api_response = self.system_groups_api.groups_system_post(content_type,
                                                                         accept,
                                                                         body=body,
                                                                         x_org_id=x_org_id)
                return api_response
            except ApiException as error:
                raise "Exception when calling SystemGroupsApi->groups_system_post: %s\n" % error
        else:
            try:
                body = jcapiv2.UserGroupPost(name=group_name)
                api_response = self.user_groups_api.groups_user_post(content_type,
                                                                     accept,
                                                                     body=body,
                                                                     x_org_id=x_org_id)
                return api_response
            except ApiException as error:
                raise "Exception when calling UserGroupsApi->groups_user_post: %s\n" % error

    def delete_group(self, name):
        """
        Delete a Jumpcloud group
        :param name: The group name
        :return: API response
        """
        group_name = name
        content_type = 'application/json'
        accept = 'application/json'
        x_org_id = ''
        group_id, group_type = self.get_group(group_name, limit=100)

        if group_id is None:
            raise UerGroupNotFoundError("User group {} not found".format(name))

        if group_type == "system_group":
            try:
                api_response = self.system_groups_api.groups_system_delete(group_id,
                                                                           content_type,
                                                                           accept,
                                                                           x_org_id=x_org_id)
                return api_response
            except ApiException as error:
                raise "Exception when calling UserGroupsApi->groups_user_delete: %s\n" % error
        else:
            try:
                api_response = self.user_groups_api.groups_user_delete(group_id,
                                                                       content_type,
                                                                       accept,
                                                                       x_org_id=x_org_id)
                return api_response
            except ApiException as error:
                raise "Exception when calling UserGroupsApi->groups_user_delete: %s\n" % error

    def bind_user_to_group(self, user_id, group_id):
        """
        Associates a Jumpcloud user to a Jumpcloud group
        :param user_id:
        :param group_id:
        :return:
        """
        content_type = 'application/json'
        accept = 'application/json'
        body = jcapiv2.UserGroupMembersReq(id=user_id,
                                           op="add",
                                           type="user")
        x_org_id = ''

        try:
            api_response = self.graph_api.graph_user_group_members_post(group_id,
                                                                        content_type,
                                                                        accept,
                                                                        body=body,
                                                                        x_org_id=x_org_id)
            return api_response
        except ApiException as error:
            raise "Exception when calling GraphApi->graph_user_group_members_post: %s\n" % error

    def bind_ldap_to_user(self, ldap_id):
        """
        Associates a Jumpcloud user to a Jumpcloud LDAP
        :param ldap_id:
        :return:
        """
        ldapserver_id = ldap_id
        content_type = 'application/json'
        accept = 'application/json'
        body = jcapiv2.GraphManagementReq()
        x_org_id = ''

        try:
            api_response = self.graph_api.graph_ldap_server_associations_post(ldapserver_id,
                                                                              content_type,
                                                                              accept,
                                                                              body=body,
                                                                              x_org_id=x_org_id)
            return api_response
        except ApiException as error:
            raise "Exception when calling \
                   GraphApi->graph_ldap_server_associations_post: %s\n" % error

    def get_group(self, group_name, limit=10):
        # pylint: disable-msg=too-many-locals
        """
        Get the jumpcloud group info from a Jumpcloud group name
        :param group_name:
        :param limit:
        :return:  The jumpcloud group id and type, NONE group is not found
        """
        content_type = 'application/json'
        accept = 'application/json'
        fields = ['[]']
        filter = ['[]']
        skip = 0
        sort = ['[]']
        x_org_id = ''

        group_id = None
        group_type = None
        try:
            results = self.groups_api.groups_list(content_type,
                                                  accept,
                                                  fields=fields,
                                                  filter=filter,
                                                  limit=limit,
                                                  skip=skip,
                                                  sort=sort,
                                                  x_org_id=x_org_id)

            groups = class_to_dict(results)

            for group in groups:
                if group['_name'] == group_name:
                    group_id = group['_id']
                    group_type = group['_type']

            return group_id, group_type
        except ApiException as error:
            raise "Exception when calling GroupsApi->groups_list: %s\n" % error



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. currentmodule:: setup.py
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

This file is used to create the package we'll publish to PyPI.
"""

import importlib.util
import os
from pathlib import Path
from setuptools import setup, find_packages
from codecs import open  # Use a consistent encoding.
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

# Get the base version from the library.  (We'll find it in the `version.py`
# file in the src directory, but we'll bypass actually loading up the library.)
vspec = importlib.util.spec_from_file_location(
  "version",
  str(Path(__file__).resolve().parent /
      'jccli'/"version.py")
)
vmod = importlib.util.module_from_spec(vspec)
vspec.loader.exec_module(vmod)
version = getattr(vmod, '__version__')

# If the environment has a build number set...
if os.getenv('buildnum') is not None:
    # ...append it to the version.
    version = "{version}.{buildnum}".format(
        version=version,
        buildnum=os.getenv('buildnum')
    )

install_requirements = [
    "click>=7.0,<8",
    "jcapiv1 @ https://github.com/TheJumpCloud/jcapi-python/archive/v3.3.0.zip#subdirectory=jcapiv1",
    "jcapiv2 @ https://github.com/TheJumpCloud/jcapi-python/archive/v3.3.0.zip#subdirectory=jcapiv2",
    "PyYaml>=5.1,<6.0"
]

setup(
    name='jccli',
    description="A Jumpcloud command line client",
    long_description=long_description,
    packages=find_packages(
        exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    version=version,
    python_requires=">=3",
    install_requires=install_requirements,
    entry_points={
        "console_scripts": [
            'jccli = jccli.cli:cli'
        ]
    },
    license='Apache Software License',  # noqa
    author='zaro0508',
    author_email='zaro0508@gmail.com',
    # Use the URL to the github repo.
    url= 'https://github.com/Sage-Bionetworks/jccli',
    download_url=(
        f'https://github.com/Sage-Bionetworks/'
        f'jccli/archive/{version}.tar.gz'
    ),
    keywords=[
        "jccli",
        "jumpcloud",
        "cli"
    ],
    # See https://PyPI.python.org/PyPI?%3Aaction=list_classifiers
    classifiers=[
      # How mature is this project? Common values are
      #   3 - Alpha
      #   4 - Beta
      #   5 - Production/Stable
      'Development Status :: 3 - Alpha',

      # Indicate who your project is intended for.
      'Intended Audience :: Developers',
      'Natural Language :: English',
      'Environment :: Console',

        # Pick your license.  (It should match "license" above.)
        # noqa
        'License :: OSI Approved :: Apache Software License',
        # noqa

      # Specify the Python versions you support here. In particular, ensure
      # that you indicate whether you support Python 2, Python 3 or both.
      'Programming Language :: Python :: 3.6',
    ],
    include_package_data=True
)



# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli.helpers.py
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

This is a set of helper methods

"""

import json

def class_to_dict(class_object):
    """
      Convert a jumpcloud class to a dictionary
    """
    result = []
    for item in class_object:
        result.append(item.__dict__)

    return result

def get_users_from_file(data_file):
    """
    Get users from a file
    :param data_file:
    :return: a list of users
    """
    try:
        with open(data_file, 'r') as file:
            jc_config = json.load(file)
            users = jc_config['users']

        return users
    except (ValueError, TypeError, IOError) as error:
        raise error

def get_groups_from_file(data_file):
    """
    Get groups from a file
    :param file:
    :return: a list of groups
    """
    try:
        with open(data_file, 'r') as file:
            jc_config = json.load(file)
            groups = jc_config['groups']

        return groups
    except (ValueError, TypeError, IOError) as error:
        raise error

def get_user_from_term(input):
    """
    Get users from a file
    :param user_file:
    :return: a list of users
    """
    try:
        print(input)
        users = json.loads(input.replace("'", '"'))

        return users
    except (ValueError, TypeError, IOError) as error:
        raise error

def get_user_from_file(user_file):
    """
    Get users from a file
    :param user_file:
    :return: a list of users
    """
    try:
        with open(user_file, 'r') as file:
            users = json.load(file)

        return users
    except (ValueError, TypeError, IOError) as error:
        raise error
